@@ -172,6 +172,27 @@ L.Draw.Polygon.include({
 
 });
 
+L.S.PolyEdit = L.Edit.Poly.extend({
+
+    _createMarker: function (latlng, index) {
+        var marker = L.Edit.Poly.prototype._createMarker.call(this, latlng, index);
+        marker._poly = this._poly;
+        marker.on('contextmenu', this._onMarkerContextMenu);
+        return marker;
+    },
+
+    _removeMarker: function (marker) {
+        marker.off('contextmenu', this._onMarkerContextMenu);
+        L.Edit.Poly.prototype._removeMarker.call(this, marker);
+    },
+
+    _onMarkerContextMenu: function (e) {
+        e._polyHandlerIndex = this._index;
+        this._poly.fire('contextmenu', e);
+    }
+
+});
+
 /* Share control */
 L.Control.Embed = L.Control.extend({
 
@@ -272,7 +293,7 @@ L.Storage.DataLayersControl = L.Control.extend({
 
         L.DomEvent
             .on(link, 'click', L.DomEvent.stop)
-            .on(link, 'click', this.openBrowser, this);
+            .on(link, 'click', map.openBrowser, map);
 
         L.DomEvent
             .on(add, 'click', L.DomEvent.stop)
@@ -301,136 +322,15 @@ L.Storage.DataLayersControl = L.Control.extend({
         }
     },
 
-    initBrowserLayout: function () {
-        this._browser_container = L.DomUtil.create('div', 'storage-browse-data');
-        var title = L.DomUtil.create('h3', 'storage-browse-title', this._browser_container);
-        var description = L.DomUtil.create('div', '', this._browser_container);
-        this._features_container = L.DomUtil.create('div', 'storage-browse-features', this._browser_container);
-        title.innerHTML = this._map.name;
-        if (this._map.description) {
-            var content = L.DomUtil.create('div', 'storage-browse-description', description);
-            content.innerHTML = L.Util.toHTML(this._map.description);
-        }
-        return this._browser_container;
-    },
-
-    openBrowser: function () {
-        if (!this._browser_container) {
-            this.initBrowserLayout();
-        }
-        this._features_container.innerHTML = '';
-        for(var idx in this._map.datalayers) {
-            this.appendOnBrowser(this._map.datalayers[idx]);
-        }
-        L.Storage.fire('ui:start', {data: {html: this._browser_container}});
-    },
-
     addDataLayer: function (datalayer) {
-        var datalayer_li = L.DomUtil.create('li', '', this._datalayers_container),
-            toggle = L.DomUtil.create('i', 'layer-toggle', datalayer_li),
-            zoom_to = L.DomUtil.create('i', 'layer-zoom_to', datalayer_li),
-            edit = L.DomUtil.create('i', "layer-edit show-on-edit", datalayer_li);
-            title = L.DomUtil.create('span', 'layer-title', datalayer_li);
+        var datalayer_li = L.DomUtil.create('li', '', this._datalayers_container);
+        datalayer.renderToolbox(datalayer_li);
+        var title = L.DomUtil.add('span', 'layer-title', datalayer_li, datalayer.options.name);
 
-        zoom_to.title = this.labels.zoomToLayer;
-        toggle.title = this.labels.toggleLayer;
         datalayer_li.id = "browse_data_toggle_" + datalayer.storage_id;
-        L.DomUtil.addClass(datalayer_li, datalayer.isVisible() ? 'on': 'off');
-
-        edit.title = this.labels.editLayer;
-        edit.href = '#';
-        if (datalayer.storage_id) {  // storage_id is null for non saved ones
-            edit.id = 'edit_datalayer_' + datalayer.storage_id;
-        }
+        L.DomUtil.classIf(datalayer_li, 'off', !datalayer.isVisible());
 
         title.innerHTML = datalayer.options.name;
-        L.DomEvent.on(toggle, 'click', datalayer.toggle, datalayer);
-        L.DomEvent.on(zoom_to, 'click', datalayer.zoomTo, datalayer);
-        L.DomEvent.on(edit, 'click', datalayer.edit, datalayer);
-        datalayer.on('hide', function () {
-            this.removeFeatures(datalayer);
-            L.DomUtil.addClass(datalayer_li, 'off');
-        }, this);
-        datalayer.on('display', function () {
-            datalayer.onceLoaded(function () {
-                this.appendOnBrowser(datalayer);
-                L.DomUtil.removeClass(datalayer_li, 'off');
-            }, this);
-        }, this);
-    },
-
-    appendOnBrowser: function (datalayer) {
-        var id = 'browse_data_datalayer_' + datalayer.storage_id,
-            self = this,
-            container = L.DomUtil.get(id), ul;
-        if (!container) {
-            container = L.DomUtil.create('div', '', this._features_container);
-            container.id = id;
-            headline = L.DomUtil.create('h5', '', container);
-            var toggle = L.DomUtil.create('i', 'layer-toggle', headline),
-                zoom_to = L.DomUtil.create('i', 'layer-zoom_to', headline),
-                edit = L.DomUtil.create('i', "layer-edit show-on-edit", headline),
-                title = L.DomUtil.add('span', '', headline, datalayer.options.name);
-            zoom_to.title = this.labels.zoomToLayer;
-            toggle.title = this.labels.toggleLayer;
-            edit.title = this.labels.editLayer;
-            L.DomEvent.on(toggle, 'click', datalayer.toggle, datalayer);
-            L.DomEvent.on(zoom_to, 'click', datalayer.zoomTo, datalayer);
-            L.DomEvent.on(edit, 'click', datalayer.edit, datalayer);
-            ul = L.DomUtil.create('ul', '', container);
-        } else {
-            ul = container.querySelector('ul');
-        }
-        L.DomUtil.classIf(container, 'off', !datalayer.isVisible());
-
-        var build = function () {
-            ul.innerHTML = "";
-            for (var j in datalayer._layers) {
-                ul.appendChild(self.addFeature(datalayer._layers[j]));
-            }
-        };
-        if (datalayer.isLoaded() && datalayer.isVisible()) {
-            build();
-        }
-        datalayer.on('dataloaded', function () {
-            if (datalayer.isVisible()) {
-                build();
-            }
-        });
-    },
-
-    addFeature: function (feature) {
-        var feature_li = L.DomUtil.create('li', feature.getClassName()),
-            zoom_to = L.DomUtil.create('i', 'feature-zoom_to', feature_li),
-            edit = L.DomUtil.create('i', 'show-on-edit feature-edit', feature_li),
-            color = L.DomUtil.create('i', 'feature-color', feature_li),
-            title = L.DomUtil.create('span', 'feature-title', feature_li),
-            symbol = feature._getIconUrl ? L.S.Icon.prototype.formatUrl(feature._getIconUrl(), feature): null;
-        zoom_to.title = L._("Bring feature to center");
-        edit.title = L._("Edit this feature");
-        title.innerHTML = feature.properties.name || '—';
-        color.style.backgroundColor = feature.getOption('color');
-        if (symbol) {
-            color.style.backgroundImage = 'url(' + symbol + ')';
-        }
-        L.DomEvent.on(zoom_to, 'click', function (e) {
-            this.bringToCenter(e, L.bind(this.view, this, {latlng: this.getCenter()}));
-        }, feature);
-        L.DomEvent.on(edit, 'click', function (e) {
-            this.edit();
-        }, feature);
-        return feature_li;
-    },
-
-    removeFeatures: function (datalayer) {
-        var el = L.DomUtil.get('browse_data_datalayer_' + datalayer.storage_id);
-        if (el) {
-            L.DomUtil.addClass(el, 'off');
-            var ul = el.querySelector('ul');
-            if (ul) {
-                ul.innerHTML = '';
-            }
-        }
     },
 
     newDataLayer: function (e) {
@@ -440,6 +340,137 @@ L.Storage.DataLayersControl = L.Control.extend({
 
 });
 
+L.Storage.DataLayer.include({
+
+    renderToolbox: function (container) {
+        var toggle = L.DomUtil.create('i', 'layer-toggle', container),
+            zoom_to = L.DomUtil.create('i', 'layer-zoom_to', container),
+            edit = L.DomUtil.create('i', "layer-edit show-on-edit", container);
+        zoom_to.title = L._('Zoom to layer extent');
+        toggle.title = L._('Show/hide layer');
+        edit.title = L._('Edit');
+        L.DomEvent.on(toggle, 'click', this.toggle, this);
+        L.DomEvent.on(zoom_to, 'click', this.zoomTo, this);
+        L.DomEvent.on(edit, 'click', this.edit, this);
+        L.DomUtil.addClass(container, this.getHidableClass());
+        L.DomUtil.classIf(container, 'off', !this.isVisible());
+    },
+
+    getLocalId: function () {
+        return this.storage_id || "tmp" + L.Util.stamp(this);
+    },
+
+    getHidableElements: function () {
+        return document.querySelectorAll('.' + this.getHidableClass());
+    },
+
+    getHidableClass: function () {
+        return 'show_with_datalayer_' + this.getLocalId();
+    },
+
+    propagateHide: function () {
+        var els = this.getHidableElements();
+        for (var i = 0; i < els.length; i++) {
+            L.DomUtil.addClass(els[i], 'off');
+        }
+    },
+
+    propagateShow: function () {
+        this.onceLoaded(function () {
+            var els = this.getHidableElements();
+            for (var i = 0; i < els.length; i++) {
+                L.DomUtil.removeClass(els[i], 'off');
+            }
+        }, this);
+    }
+
+});
+
+L.Storage.DataLayer.addInitHook(function () {
+    this.on('hide', this.propagateHide);
+    this.on('show', this.propagateShow);
+    this.propagateShow();
+});
+
+
+L.Storage.Map.include({
+
+    _openBrowser: function () {
+        var browserContainer = L.DomUtil.create('div', 'storage-browse-data'),
+            title = L.DomUtil.add('h3', 'storage-browse-title', browserContainer, this.options.name),
+            filter = L.DomUtil.create('input', '', browserContainer),
+            filterValue = '',
+            featuresContainer = L.DomUtil.create('div', 'storage-browse-features', browserContainer);
+        filter.type = 'text';
+        filter.placeholder = L._('Filter…');
+
+        var addFeature = function (feature) {
+            var feature_li = L.DomUtil.create('li', feature.getClassName() + ' feature'),
+                zoom_to = L.DomUtil.create('i', 'feature-zoom_to', feature_li),
+                edit = L.DomUtil.create('i', 'show-on-edit feature-edit', feature_li),
+                color = L.DomUtil.create('i', 'feature-color', feature_li),
+                title = L.DomUtil.create('span', 'feature-title', feature_li),
+                symbol = feature._getIconUrl ? L.S.Icon.prototype.formatUrl(feature._getIconUrl(), feature): null;
+            zoom_to.title = L._("Bring feature to center");
+            edit.title = L._("Edit this feature");
+            title.innerHTML = feature.getDisplayName() || '—';
+            color.style.backgroundColor = feature.getOption('color');
+            if (symbol) {
+                color.style.backgroundImage = 'url(' + symbol + ')';
+            }
+            L.DomEvent.on(zoom_to, 'click', function (e) {
+                this.bringToCenter(e, L.bind(this.view, this, {latlng: this.getCenter()}));
+            }, feature);
+            L.DomEvent.on(edit, 'click', function (e) {
+                this.edit();
+            }, feature);
+            return feature_li;
+        };
+
+        var append = function (datalayer) {
+            var container = L.DomUtil.create('div', datalayer.getHidableClass(), featuresContainer),
+                headline = L.DomUtil.create('h5', '', container);
+            container.id = 'browse_data_datalayer_' + datalayer.storage_id;
+            datalayer.renderToolbox(headline);
+            var title = L.DomUtil.add('span', '', headline, datalayer.options.name),
+                ul = L.DomUtil.create('ul', '', container);
+            L.DomUtil.classIf(container, 'off', !datalayer.isVisible());
+
+            var build = function () {
+                ul.innerHTML = "";
+                datalayer.eachFeature(function (feature) {
+                    if (filterValue && (feature.getDisplayName() || "").toLowerCase().indexOf(filterValue) === -1) return;
+                    ul.appendChild(addFeature(feature));
+                });
+            };
+            build();
+            datalayer.on('datachanged', build);
+            L.Storage.once('ui:end', function () {
+                datalayer.off('datachanged', build);
+            });
+            L.Storage.once('ui:ready', function () {
+                L.Storage.once('ui:start', function () {
+                    datalayer.off('datachanged', build);
+                });
+            });
+        };
+
+        var appendAll = function () {
+            featuresContainer.innerHTML = '';
+            filterValue = filter.value;
+            this.eachDataLayer(function (datalayer) {
+                append(datalayer);
+            });
+        };
+        L.bind(appendAll, this)();
+        L.DomEvent.on(filter, 'input', appendAll, this);
+        L.Storage.fire('ui:start', {data: {html: browserContainer}});
+    }
+
+});
+
+
+
 L.Storage.TileLayerControl = L.Control.extend({
     options: {
         position: 'topleft'
@@ -604,7 +635,7 @@ L.Storage.JumpToLocationControl = L.Control.extend({
                     L.DomUtil.removeClass(link, 'loading');
                     if (data.length > 0 && data[0].lon && data[0].lat) {
                         map.panTo([data[0].lat, data[0].lon]);
-                        map.setZoom(15);
+                        map.setZoom(map.options.jumpToZoom);
                     }
                     else {
                         L.S.fire('ui:alert', {content: L._('Sorry, no location found for {location}', {location: search_terms})});
@@ -689,11 +720,8 @@ L.S.ContextMenu = L.Map.ContextMenu.extend({
 
 });
 
-
+L.Control.MeasureControl.TITLE = L._('Measure distances');
 L.S.MeasureControl = L.Control.MeasureControl.extend({
-    statics: {
-        TITLE: L._('Measure distances')
-    },
 
     onAdd: function (map) {
         L.Control.MeasureControl.prototype.onAdd.call(this, map);
@@ -705,4 +733,51 @@ L.S.MeasureControl = L.Control.MeasureControl.extend({
         return this._container;
     }
 
+});
+
+L.S.IframeExporter = L.Class.extend({
+    includes: [L.Mixin.Events],
+
+    options: {
+        includeFullScreenLink: true,
+        currentView: false
+    },
+
+    queryString : {
+        scaleControl: false,
+        miniMap: false,
+        scrollWheelZoom: false,
+        zoomControl: true,
+        allowEdit: false,
+        moreControl: true,
+        datalayersControl: true,
+        onLoadPanel: "none"
+    },
+
+    dimensions: {
+        width: '100%',
+        height: '300px'
+    },
+
+    initialize: function (map) {
+        this.map = map;
+        this.baseUrl = window.location.protocol + '//' + window.location.host + window.location.pathname;
+        // Use map default, not generic default
+        this.queryString.onLoadPanel = this.map.options.onLoadPanel;
+    },
+
+    getMap: function () {
+        return this.map;
+    },
+
+    build: function () {
+        var currentView = this.options.currentView ? window.location.hash : '',
+            iframeUrl = this.baseUrl + '?' + L.S.Xhr.buildQueryString(this.queryString) + currentView,
+            code = '<iframe width="' + this.dimensions.width + '" height="' + this.dimensions.height + '" frameBorder="0" src="' + iframeUrl +'"></iframe>';
+        if (this.options.includeFullScreenLink) {
+            code += '<p><a href="' + this.baseUrl + '">' + L._('See full screen') + '</a></p>';
+        }
+        return code;
+    }
+
 });
\ No newline at end of file
