@@ -51,17 +51,18 @@ L.Storage.FeatureMixin = {
             var win = window.open(this.properties._storage_options.outlink);
             return;
         }
-        this.populatePopup();
+        this.attachPopup();
         this.openPopup(e.latlng);
     },
 
     edit: function(e) {
         if(!this.map.editEnabled || this.isReadOnly()) return;
-        this.map.editedFeature = this;
         var self = this,
             container = L.DomUtil.create('div'), form ;
 
-        var builder = new L.S.FormBuilder(this, ['datalayer']);
+        var builder = new L.S.FormBuilder(this, ['datalayer'], {
+            callback: function () {this.edit(e);}  // removeLayer step will close the edit panel, let's reopen it
+        });
         container.appendChild(builder.build());
 
         var properties = [];
@@ -73,7 +74,12 @@ L.Storage.FeatureMixin = {
         // We always want name and description for now (properties management to come)
         properties.unshift('properties.description');
         properties.unshift('properties.name');
-        builder = new L.S.FormBuilder(this, properties, {id: 'storage-feature-properties'});
+        builder = new L.S.FormBuilder(this, properties,
+            {
+                id: 'storage-feature-properties',
+                callback: this.resetLabel
+            }
+        );
         form = builder.build();
         container.appendChild(form);
         var options_fields = this.getAdvancedOptions();
@@ -88,6 +94,7 @@ L.Storage.FeatureMixin = {
         var advancedActions = L.DomUtil.createFieldset(container, L._('Advanced actions'));
         this.getAdvancedEditActions(advancedActions);
         L.S.fire('ui:start', {data: {html: container}});
+        this.map.editedFeature = this;
         this.bringToCenter(e);
     },
 
@@ -96,6 +103,7 @@ L.Storage.FeatureMixin = {
         deleteLink.href = "#";
         deleteLink.innerHTML = L._('Delete');
         L.DomEvent.on(deleteLink, "click", function (e) {
+            L.DomEvent.stop(e);
             if (this.confirmDelete()) {
                 L.S.fire('ui:end');
             }
@@ -104,33 +112,11 @@ L.Storage.FeatureMixin = {
 
     endEdit: function () {},
 
-    defaultPopupTemplate: function (container) {
-        if (this.properties.description) {
-            var content = L.DomUtil.create('p', '', container);
-            content.innerHTML = L.Util.toHTML(this.properties.description);
-        }
-
+    getDisplayName: function () {
+        return this.properties.name || this.properties.title || this.datalayer.options.name;
     },
 
-    tablePopupTemplate: function (container) {
-        var table = L.DomUtil.create('table', '', container);
-
-        var addRow = function (key, value) {
-            var tr = L.DomUtil.create('tr', '', table);
-            L.DomUtil.add('th', '', tr, key);
-            L.DomUtil.add('td', '', tr, value);
-        };
-
-        for (var key in this.properties) {
-            if (typeof this.properties[key] === "object") {
-                continue;
-            }
-            // TODO, manage links (url, mailto, wikipedia...)
-            addRow(key, L.Util.escapeHTML(this.properties[key]));
-        }
-    },
-
-    displayPopupFooter: function () {
+    hasPopupFooter: function () {
         if (L.Browser.ielt9) {
             return false;
         }
@@ -140,48 +126,13 @@ L.Storage.FeatureMixin = {
         return this.map.options.displayPopupFooter;
     },
 
-    populatePopup: function () {
-        var container = L.DomUtil.create('div', '');
-        if (this.properties.name) {
-            L.DomUtil.add('h4', '', container, L.Util.escapeHTML(this.properties.name));
-        }
-        var content = L.DomUtil.create('div', 'storage-popup-content', container),
-            template = this.getOption('popupTemplate');
-        if (template === "table") {
-            this.tablePopupTemplate(content);
-        } else {
-            this.defaultPopupTemplate(content);
-        }
-        if (this.displayPopupFooter()) {
-            var footer = L.DomUtil.create('ul', 'storage-popup-footer', container),
-                previous_li = L.DomUtil.create('li', 'previous', footer),
-                zoom_li = L.DomUtil.create('li', 'zoom', footer),
-                next_li = L.DomUtil.create('li', 'next', footer),
-                next = this.getNext(),
-                prev = this.getPrevious();
-            if (next) {
-                next_li.title = L._("Go to «{feature}»", {feature: next.properties.name});
-            }
-            if (prev) {
-                previous_li.title = L._("Go to «{feature}»", {feature: prev.properties.name});
-            }
-            zoom_li.title = L._("Zoom to this feature");
-            L.DomEvent.on(next_li, 'click', function (e) {
-                if (next) {
-                    next.bringToCenter(e, function () {next.view(next.getCenter());});
-                }
-            }, this);
-            L.DomEvent.on(previous_li, 'click', function (e) {
-                if (prev) {
-                    prev.bringToCenter(e, function () {prev.view(prev.getCenter());});
-                }
-            }, this);
-            L.DomEvent.on(zoom_li, 'click', function (e) {
-                this.map._zoom = 16;  // Do not hardcode this
-                this.bringToCenter();
-            }, this);
-        }
-        this.bindPopup(container);
+    getPopupClass: function () {
+        return L.Storage.Popup[this.getOption('popupTemplate')] || L.Storage.Popup;
+    },
+
+    attachPopup: function () {
+        var Class = this.getPopupClass();
+        this.bindPopup(new Class(this));
     },
 
     confirmDelete: function () {
@@ -243,6 +194,9 @@ L.Storage.FeatureMixin = {
         else if (this.datalayer && this.usableOption(this.datalayer.options, option)) {
             value = this.datalayer.options[option];
         }
+        else if (this.usableOption(this.map.options, option)) {
+            value = this.map.options[option];
+        }
         else {
             value = this.map.getDefaultOption(option);
         }
@@ -271,11 +225,20 @@ L.Storage.FeatureMixin = {
         return this.datalayer.getPreviousFeature(this);
     },
 
+    cloneProperties: function () {
+        var properties = L.extend({}, this.properties);
+        properties._storage_options = L.extend({}, properties._storage_options);
+        if (Object.keys && Object.keys(properties._storage_options).length === 0) {
+            delete properties._storage_options;  // It can make a difference on big data sets
+        }
+        return properties;
+    },
+
     toGeoJSON: function () {
         return {
             type: "Feature",
             geometry: this.geometry(),
-            properties: L.extend({}, this.properties)
+            properties: this.cloneProperties()
         };
     },
 
@@ -285,7 +248,7 @@ L.Storage.FeatureMixin = {
 
     _showContextMenu: function (e) {
         var pt = this.map.mouseEventToContainerPoint(e.originalEvent);
-        this.map.contextmenu.showAt(pt, {relatedTarget: this});
+        this.map.contextmenu.showAt(pt, {relatedTarget: this, _polyHandlerIndex: e._polyHandlerIndex});
     },
 
     makeDirty: function () {
@@ -296,30 +259,33 @@ L.Storage.FeatureMixin = {
         return this._map;
     },
 
-    getContextMenuItems: function () {
+    getContextMenuItems: function (e) {
         var items = [];
         if (this._map.editEnabled && !this.isReadOnly()) {
-            items = items.concat(this.getEditContextMenuItems());
+            items = items.concat(this.getEditContextMenuItems(e));
         }
         return items;
     },
 
-    getEditContextMenuItems: function () {
+    getEditContextMenuItems: function (e) {
         return ['-',
             {
                 text: L._('Edit this feature'),
                 callback: this.edit,
-                context: this
+                context: this,
+                iconCls: 'storage-edit'
             },
             {
                 text: L._("Edit feature's layer"),
                 callback: this.datalayer.edit,
-                context: this.datalayer
+                context: this.datalayer,
+                iconCls: 'storage-edit'
             },
             {
                 text: L._('Delete this feature'),
                 callback: this.confirmDelete,
-                context: this
+                context: this,
+                iconCls: 'storage-delete'
             }
         ];
     },
@@ -353,6 +319,17 @@ L.Storage.FeatureMixin = {
         if (this.tooltip) {
             this.tooltip.dispose();
         }
+    },
+
+    resetLabel: function () {
+        if (this.label) {
+            this.hideLabel();
+            delete this.label;
+        }
+        if (this.getOption('showLabel') && this.properties.name) {
+            this.bindLabel(L.Util.escapeHTML(this.properties.name), {noHide: true});
+            this.showLabel();
+        }
     }
 
 };
@@ -372,7 +349,9 @@ L.Storage.Marker = L.Marker.extend({
             this.edit(e);
         }, this);
         this.on("click", this._onClick);
-        this.on("mouseover", this._enableDragging);
+        if (!this.isReadOnly()) {
+            this.on("mouseover", this._enableDragging);
+        }
         this.on("mouseout", this._onMouseOut);
         this._popupHandlersAdded = true; // prevent Leaflet from binding event on bindPopup
     },
@@ -422,6 +401,7 @@ L.Storage.Marker = L.Marker.extend({
     _initIcon: function () {
         this.options.icon = this.getIcon();
         L.Marker.prototype._initIcon.call(this);
+        this.resetLabel();
     },
 
     disconnectFromDataLayer: function (datalayer) {
@@ -445,25 +425,11 @@ L.Storage.Marker = L.Marker.extend({
         if (typeof name === "undefined") {
             name = "icon";
         }
-        var url = null;
-        if (this.properties._storage_options[name + 'Url']) {
-            url = this.properties._storage_options[name + 'Url'];
-        }
-        else if(this.datalayer && this.datalayer.options[name + 'Url']) {
-            url = this.datalayer.options[name + 'Url'];
-        }
-        return url;
+        return this.getOption(name + 'Url');
     },
 
     getIconClass: function () {
-        var iconClass = this.map.getDefaultOption('iconClass');
-        if (this.properties._storage_options.iconClass) {
-            iconClass = this.properties._storage_options.iconClass;
-        }
-        else if (this.datalayer) {
-            iconClass = this.datalayer.getIconClass();
-        }
-        return iconClass;
+        return this.getOption('iconClass');
     },
 
     getIcon: function () {
@@ -491,7 +457,8 @@ L.Storage.Marker = L.Marker.extend({
             'properties._storage_options.color',
             'properties._storage_options.iconClass',
             'properties._storage_options.iconUrl',
-            'properties._storage_options.popupTemplate'
+            'properties._storage_options.popupTemplate',
+            ['properties._storage_options.showLabel', {handler: 'CheckBox', helpText: L._('Add a permanent label')}]
         ];
     },
 
@@ -590,7 +557,26 @@ L.Storage.PathMixin = {
     onAdd: function (map) {
         this._container = null;
         this._setStyleOptions();
+        if (this.map._controls.measureControl) {
+            this.map._controls.measureControl.handler.on('enabled', this.showMeasureTooltip, this);
+            this.map._controls.measureControl.handler.on('disabled', function () {
+                this.removeTooltip();
+            }, this);
+        }
         this.parentClass.prototype.onAdd.call(this, map);
+        if (this.editing && this.editing.enabled()) {
+            this.editing.addHooks();
+        }
+    },
+
+    onRemove: function (map) {
+        if (this.map._controls.measureControl) {
+            this.map._controls.measureControl.handler.off('enabled', this.showMeasureTooltip, this);
+        }
+        if (this.editing && this.editing.enabled()) {
+            this.editing.removeHooks();
+        }
+        L.S.FeatureMixin.onRemove.call(this, map);
     },
 
     getCenter: function () {
@@ -608,23 +594,22 @@ L.Storage.PathMixin = {
         }
     },
 
+    showMeasureTooltip: function () {
+        if (this.datalayer.isVisible()) {
+            this.showTooltip({text: this.getMeasure()});
+        }
+    },
+
     addInteractions: function () {
         L.Storage.FeatureMixin.addInteractions.call(this);
         this.on("dragend", this.edit);
         this.on("click", this._onClick);
-        this.on("dblclick", this._toggleEditing);
+        if (!this.isReadOnly()) {
+            this.on("dblclick", this._toggleEditing);
+        }
         this.on("mouseover", this._onMouseOver);
         this.on("edit", this.makeDirty);
-        if (this.map._controls.measureControl) {
-            this.map._controls.measureControl.handler.on('enabled', function () {
-                if (this.datalayer.isVisible()) {
-                    this.showTooltip({text: this.getMeasure()});
-                }
-            }, this);
-            this.map._controls.measureControl.handler.on('disabled', function () {
-                this.removeTooltip();
-            }, this);
-        }
+        this.editing = new L.S.PolyEdit(this);
     }
 
 };
@@ -640,16 +625,9 @@ L.Storage.Polyline = L.Polyline.extend({
 
     geometry: function() {
         /* Return a GeoJSON geometry Object */
-        var latlngs = this.getLatLngs(), coords = [];
-        for(var i = 0, len = latlngs.length; i < len; i++) {
-            coords.push([
-                latlngs[i].lng,
-                latlngs[i].lat
-            ]);
-        }
         return {
             type: "LineString",
-            coordinates: coords
+            coordinates: L.Util.latLngsForGeoJSON(this.getLatLngs())
         };
     },
 
@@ -669,13 +647,35 @@ L.Storage.Polyline = L.Polyline.extend({
         return L.GeometryUtil.readableDistance(distance, true);
     },
 
-    getEditContextMenuItems: function () {
-        var items = L.Storage.FeatureMixin.getEditContextMenuItems.call(this);
+    getEditContextMenuItems: function (e) {
+        var items = L.Storage.FeatureMixin.getEditContextMenuItems.call(this),
+            handlerClicked = e._polyHandlerIndex;
         items.push({
             text: L._('Transform to polygon'),
             callback: this.toPolygon,
             context: this
         });
+        if (this.map.editedFeature
+            && this.map.editedFeature instanceof L.Storage.Polyline
+            && this.map.editedFeature !== this) {
+            items.push({
+                text: L._('Merge geometry with edited feature'),
+                callback: function () {
+                    this.mergeInto(this.map.editedFeature);
+                    this.map.editedFeature.editing.updateMarkers();
+                },
+                context: this
+            });
+        }
+        if (handlerClicked && handlerClicked !== 0 && handlerClicked !== this._latlngs.length-1) {
+            items.push({
+                text: L._('Split line'),
+                callback: function () {
+                    this.splitAt(handlerClicked);
+                },
+                context: this
+            });
+        }
         return items;
     },
 
@@ -694,6 +694,64 @@ L.Storage.Polyline = L.Polyline.extend({
         toPolygon.href = "#";
         toPolygon.innerHTML = L._('Transform to polygon');
         L.DomEvent.on(toPolygon, "click", this.toPolygon, this);
+    },
+
+    mergeInto: function (other) {
+        if (!other instanceof L.Storage.Polyline) return;
+        var otherLatlngs = other.getLatLngs(),
+            otherLeft = otherLatlngs[0],
+            otherRight = otherLatlngs[otherLatlngs.length-1],
+            thisLatlngs = this.getLatLngs(),
+            thisLeft = thisLatlngs[0],
+            thisRight = thisLatlngs[thisLatlngs.length-1],
+            l2ldistance = otherLeft.distanceTo(thisLeft),
+            l2rdistance = otherLeft.distanceTo(thisRight),
+            r2ldistance = otherRight.distanceTo(thisLeft),
+            r2rdistance = otherRight.distanceTo(thisRight),
+            toMerge;
+        if (l2rdistance < Math.min(l2ldistance, r2ldistance, r2rdistance)) {
+            toMerge = [thisLatlngs, otherLatlngs];
+        } else if (r2ldistance < Math.min(l2ldistance, l2rdistance, r2rdistance)) {
+            toMerge = [otherLatlngs, thisLatlngs];
+        } else if (r2rdistance < Math.min(l2ldistance, l2rdistance, r2ldistance)) {
+            thisLatlngs.reverse();
+            toMerge = [otherLatlngs, thisLatlngs];
+        } else {
+            thisLatlngs.reverse();
+            toMerge = [thisLatlngs, otherLatlngs];
+        }
+        var a = toMerge[0],
+            b = toMerge[1],
+            p1 = this.map.latLngToContainerPoint(a[a.length-1]),
+            p2 = this.map.latLngToContainerPoint(b[0]),
+            tolerance = 5; // px on screen
+        if (Math.abs(p1.x - p2.x) <= tolerance && Math.abs(p1.y - p2.y) <= tolerance) {
+            a.pop();
+        }
+        other.setLatLngs(a.concat(b));
+        this.del();
+    },
+
+    splitAt: function (index) {
+        if (index === 0 || index === this._latlngs.length-1) return;
+        var latlngs = this.getLatLngs(),
+            thisLatlngs = latlngs.slice(0, index+1),
+            otherLatlngs = latlngs.slice(index);
+        var geojson = {
+            type: 'Feature',
+            geometry: {
+                type: 'LineString',
+                coordinates: L.Util.latLngsForGeoJSON(otherLatlngs)
+            },
+            properties: this.cloneProperties()
+        };
+        this.setLatLngs(thisLatlngs);
+        this.isDirty = true;
+        if (this.editing && this.editing.enabled()) {
+            this.editing.updateMarkers();
+        }
+        var other = this.datalayer.geojsonToFeatures(geojson);
+        return other;
     }
 
 });
@@ -703,10 +761,11 @@ L.Storage.Polygon = L.Polygon.extend({
     includes: [L.Storage.FeatureMixin, L.Storage.PathMixin, L.Mixin.Events],
 
     geometry: function() {
+        // TODO: add test!
         /* Return a GeoJSON geometry Object */
         /* see: https://github.com/CloudMade/Leaflet/issues/1135 */
         /* and: https://github.com/CloudMade/Leaflet/issues/712 */
-        var latlngs = this.getLatLngs(), coords = [], closingPoint = latlngs[0];
+        var latlngs = this.getLatLngs().slice(0), coords = [], closingPoint = latlngs[0];
         latlngs.push(closingPoint);  // Artificially create a LinearRing
         for(var i = 0, len = latlngs.length; i < len; i++) {
             coords.push([
