@@ -77,7 +77,7 @@ L.S.ElementHelper.Textarea = L.S.ElementHelper.extend({
     onKeyPress: function (e) {
         var key = e.keyCode,
             ENTER = 13;
-        if (key == ENTER && (e.shiftKey || e.ctrlKey)) {
+        if (key == ENTER && (e.shiftKey || e.ctrlKey)) {
             L.DomEvent.stop(e);
             L.S.fire('ui:end');
         }
@@ -321,6 +321,7 @@ L.S.ElementHelper.SelectAbstract = L.S.ElementHelper.extend({
     build: function () {
         this.select = L.DomUtil.create('select', '', this.form);
         this.select.name = this.name;
+        this.validValues = [];
         this.buildOptions();
         L.DomEvent.on(this.select, 'change', this.sync, this);
     },
@@ -342,6 +343,7 @@ L.S.ElementHelper.SelectAbstract = L.S.ElementHelper.extend({
     },
 
     buildOption: function (value, label) {
+        this.validValues.push(value);
         var option = L.DomUtil.create('option', '', this.select);
         option.value = value;
         option.innerHTML = label;
@@ -352,8 +354,22 @@ L.S.ElementHelper.SelectAbstract = L.S.ElementHelper.extend({
 
     value: function () {
         return this.select[this.select.selectedIndex].value;
+    },
+
+    getDefault: function () {
+        return this.getOptions()[0][0];
+    },
+
+    toJS: function () {
+        var value = this.value();
+        if (this.validValues.indexOf(value) !== -1) {
+            return value;
+        } else {
+            return this.getDefault();
+        }
     }
 
+
 });
 
 L.S.ElementHelper.IconClassSwitcher = L.S.ElementHelper.SelectAbstract.extend({
@@ -364,21 +380,7 @@ L.S.ElementHelper.IconClassSwitcher = L.S.ElementHelper.SelectAbstract.extend({
         ["Circle", L._('Circle')],
         ["Drop", L._('Drop')],
         ["Ball", L._('Ball')]
-    ],
-
-    toJS: function () {
-        var value = this.value();
-        switch(value) {
-            case "Default":
-            case "Circle":
-            case "Drop":
-            case "Ball":
-                break;
-            default:
-                value = undefined;
-        }
-        return value;
-    }
+    ]
 
 });
 
@@ -386,30 +388,37 @@ L.S.ElementHelper.PopupTemplate = L.S.ElementHelper.SelectAbstract.extend({
 
     selectOptions: [
         [undefined, L._('inherit')],
-        ["default", L._('name and description')],
-        ["table", L._('table')]
+        ["Default", L._('Name and description')],
+        ["Large", L._('Name and description (large)')],
+        ["Table", L._('Table')],
+        ["GeoRSSImage", L._('GeoRSS (title + image)')],
+        ["GeoRSSLink", L._('GeoRSS (only link)')]
     ],
 
     toJS: function () {
-        var value = this.value();
-        switch(value) {
-            case "default":
-            case "table":
-                break;
-            default:
-                value = undefined;
-        }
+        var value = L.S.ElementHelper.SelectAbstract.prototype.toJS.apply(this);
+        if (value === "table") { value = "Table"; }
         return value;
     }
 
 });
 
+L.S.ElementHelper.LayerTypeChooser = L.S.ElementHelper.SelectAbstract.extend({
+
+    selectOptions: [
+        ["Default", L._('Default')],
+        ["Cluster", L._('Clustered')],
+        ["Heat", L._('Heatmap')],
+    ]
+
+});
+
 L.S.ElementHelper.DataLayerSwitcher = L.S.ElementHelper.SelectAbstract.extend({
 
     getOptions: function () {
         var options = [];
         this.map.eachDataLayer(function (datalayer) {
-            if(!datalayer.isRemoteLayer()) {
+            if(datalayer.isLoaded() && !datalayer.isRemoteLayer() && datalayer.isBrowsable()) {
                 options.push([L.stamp(datalayer), datalayer.getName()]);
             }
         });
@@ -430,6 +439,16 @@ L.S.ElementHelper.DataLayerSwitcher = L.S.ElementHelper.SelectAbstract.extend({
 
 });
 
+L.S.ElementHelper.onLoadPanel = L.S.ElementHelper.SelectAbstract.extend({
+
+    selectOptions: [
+        ['none', L._('None')],
+        ['caption', L._('Caption')],
+        ['databrowser', L._('Data browser')]
+    ]
+
+});
+
 
 L.S.ElementHelper.DataFormat = L.S.ElementHelper.SelectAbstract.extend({
 
@@ -439,22 +458,9 @@ L.S.ElementHelper.DataFormat = L.S.ElementHelper.SelectAbstract.extend({
         ['osm', 'osm'],
         ['csv', 'csv'],
         ['gpx', 'gpx'],
-        ['kml', 'kml']
-    ],
-
-    toJS: function () {
-        var value = this.value();
-        switch(value) {
-            case "osm":
-            case "gpx":
-            case "csv":
-            case "kml":
-                break;
-            default:
-                value = 'geojson';
-        }
-        return value;
-    }
+        ['kml', 'kml'],
+        ['georss', 'georss']
+    ]
 
 });
 
@@ -528,14 +534,14 @@ L.S.ElementHelper.IconUrl = L.S.ElementHelper.Input.extend({
         if (this.value() && this.value().indexOf('{') == -1) { // Do not try to render URL with variables
             var img = L.DomUtil.create('img', '', L.DomUtil.create('div', "storage-icon-choice", this.buttonsContainer));
             img.src = this.value();
-            L.DomEvent.on(img, "click", this.fetch, this);
+            L.DomEvent.on(img, "click", this.fetchIconList, this);
         }
         this.button = L.DomUtil.create('a', '', this.buttonsContainer);
         this.button.innerHTML = this.value() ? L._('Change symbol') : L._('Add symbol');
         this.button.href = "#";
         L.DomEvent
             .on(this.button, "click", L.DomEvent.stop)
-            .on(this.button, "click", this.fetch, this);
+            .on(this.button, "click", this.fetchIconList, this);
     },
 
     addIconPreview: function (pictogram) {
@@ -566,7 +572,7 @@ L.S.ElementHelper.IconUrl = L.S.ElementHelper.Input.extend({
         this.createButtonsBar();
     },
 
-    fetch: function (e) {
+    fetchIconList: function (e) {
         this.map.get(this.map.options.urls.pictogram_list_json, {
             callback: function (data) {
                 this.pictogramsContainer.innerHTML = "";
@@ -671,7 +677,7 @@ L.Storage.FormBuilder = L.Class.extend({
         this.helpers[field] = helper;
         helper.on('synced', function () {
             if (this.options.callback) {
-                this.options.callback.call(this.callbackContext || this.obj, field);
+                this.options.callback.call(this.options.callbackContext || this.obj, field);
             }
             this.obj.fire('synced', {field: field});
         }, this);
@@ -756,7 +762,16 @@ L.Storage.FormBuilder = L.Class.extend({
         iconClass: {handler: 'IconClassSwitcher', label: L._('type of icon')},
         iconUrl: {handler: 'IconUrl', label: L._('symbol of the icon')},
         popupTemplate: {handler: 'PopupTemplate', label: L._('template to use for the popup')},
-        datalayer: {handler: 'DataLayerSwitcher', label: L._('Choose the layer of the feature')}
+        datalayer: {handler: 'DataLayerSwitcher', label: L._('Choose the layer of the feature')},
+        moreControl: {handler: 'CheckBox', helpText: L._("Do you want to display the 'more' control?")},
+        datalayersControl: {handler: 'CheckBox', helpText: L._("Do you want to display the data layers control?")},
+        zoomControl: {handler: 'CheckBox', helpText: L._("Do you want to display zoom control?")},
+        scrollWheelZoom: {handler: 'CheckBox', helpText: L._("Allow scroll wheel zoom?")},
+        miniMap: {handler: 'CheckBox', helpText: L._("Do you want to display a minimap?")},
+        scaleControl: {handler: 'CheckBox', helpText: L._("Do you want to display the scale control?")},
+        onLoadPanel: {handler: 'onLoadPanel', helpText: L._("Do you want to display a panel on load?")},
+        displayPopupFooter: {handler: 'CheckBox', helpText: L._("Do you want to display popup footer?")},
+        captionBar: {handler: 'CheckBox', helpText: L._("Do you want to display a caption bar?")}
     }
 
 });
\ No newline at end of file
