@@ -25,11 +25,9 @@ L.Map.mergeOptions({
     editInOSMControlOptions: false,
     scaleControl: true,
     miniMap: false,
-    displayCaptionOnLoad: false,
     name: '',
     description: '',
     displayPopupFooter: false,
-    displayDataBrowserOnLoad: false,
     demoTileInfos: {s:'a', z:9, x:265, y:181},
     tilelayersControl: true,
     licences: [],
@@ -40,7 +38,10 @@ L.Map.mergeOptions({
     limitBounds: {},
     importPresets: [
         // {url: 'http://localhost:8019/en/datalayer/1502/', label: 'Simplified World Countries', format: 'geojson'}
-    ]
+    ],
+    moreControl: true,
+    jumpToZoom: 15,
+    captionBar: false
 });
 
 L.Storage.Map.include({
@@ -52,6 +53,7 @@ L.Storage.Map.include({
         }
         var zoomControl = typeof geojson.properties.zoomControl !== "undefined" ? geojson.properties.zoomControl : true;
         geojson.properties.zoomControl = false;
+        L.Util.setBooleanFromQueryString(geojson.properties, 'scrollWheelZoom');
         L.Map.prototype.initialize.call(this, el, geojson.properties);
         this.initLoader();
         this.name = this.options.name;
@@ -60,17 +62,15 @@ L.Storage.Map.include({
         if (geojson.geometry) {
             this.options.center = geojson.geometry;
         }
-        if (typeof this.options.moreControl === "undefined") {
-            this.options.moreControl = true;
-        }
         this.options.zoomControl = zoomControl;
         this.overrideBooleanOptionFromQueryString("zoomControl");
         this.overrideBooleanOptionFromQueryString("moreControl");
         this.overrideBooleanOptionFromQueryString("miniMap");
-        this.overrideBooleanOptionFromQueryString("scrollWheelZoom");
         this.overrideBooleanOptionFromQueryString("scaleControl");
         this.overrideBooleanOptionFromQueryString("allowEdit");
         this.overrideBooleanOptionFromQueryString("datalayersControl");
+        this.overrideBooleanOptionFromQueryString("displayDataBrowserOnLoad");
+        this.overrideBooleanOptionFromQueryString("displayCaptionOnLoad");
         if (L.Browser.ielt9) {
             // TODO include ie9
             this.options.allowEdit = false;
@@ -97,8 +97,8 @@ L.Storage.Map.include({
         if (this.options.hash) {
             this.addHash();
         }
-        this.handleLimitBounds();
         this.initCenter();
+        this.handleLimitBounds();
 
         this.initTileLayers(this.options.tilelayers);
         this.initControls();
@@ -114,10 +114,23 @@ L.Storage.Map.include({
             }
         }
         if (this.options.displayCaptionOnLoad) {
-            this.displayCaption();
+            // Retrocompat
+            if (!this.options.onLoadPanel) {
+                this.options.onLoadPanel = "caption";
+            }
+            delete this.options.displayCaptionOnLoad;
         }
-        else if (this.options.displayDataBrowserOnLoad && this.options.datalayersControl) {
+        if (this.options.displayDataBrowserOnLoad) {
+            // Retrocompat
+            if (!this.options.onLoadPanel) {
+                this.options.onLoadPanel = "databrowser";
+            }
+            delete this.options.displayDataBrowserOnLoad;
+        }
+        if (this.options.onLoadPanel === "databrowser") {
             this.openBrowser();
+        } else if (this.options.onLoadPanel === "caption") {
+            this.displayCaption();
         }
 
         L.Storage.on('ui:closed', function () {
@@ -161,9 +174,15 @@ L.Storage.Map.include({
             var datalayer = this._createDataLayer({name: L._('Layer 1')});
             datalayer.connectToMap();
             this.enableEdit();
+            var dataUrl = L.Util.queryString('dataUrl', null),
+                dataFormat = L.Util.queryString('dataFormat', 'geojson');
+            if (dataUrl) {
+                datalayer.importFromUrl(dataUrl, dataFormat);
+            }
         }
 
         this.help = new L.Storage.Help(this);
+        this.initCaptionBar();
         if (this.options.allowEdit) {
             // Layer for items added by users
             this.on('draw:created', function (e) {
@@ -184,7 +203,7 @@ L.Storage.Map.include({
             this.on('draw:start', function (e) {
                 e.layer.isDirty = true;
             });
-            L.Storage.on('ui:end', function (e) {
+            L.Storage.on('ui:end ui:start', function (e) {
                 this.editedFeature = null;
             }, this);
             this.initEditBar();
@@ -260,8 +279,9 @@ L.Storage.Map.include({
             delete this._controls[i];
         }
 
+        L.DomUtil.classIf(document.body, 'storage-caption-bar-enabled', this.options.captionBar);
         if (this.options.zoomControl) {
-            this._controls.zoomControl = (new L.Control.Zoom()).addTo(this);
+            this._controls.zoomControl = (new L.Control.Zoom({zoomInTitle: L._('Zoom in'), zoomOutTitle: L._('Zoom out')})).addTo(this);
         }
         if (this.options.datalayersControl) {
             this._controls.datalayersControl = new L.Storage.DataLayersControl().addTo(this);
@@ -280,7 +300,11 @@ L.Storage.Map.include({
             this._controls.jumpToLocationControl = (new L.Storage.JumpToLocationControl()).addTo(this);
             this._controls.embedControl = (new L.Control.Embed(this, this.options.embedOptions)).addTo(this);
             this._controls.tilelayersControl = new L.Storage.TileLayerControl().addTo(this);
-            this._controls.editInOSMControl = (new L.Control.EditInOSM({position: 'topleft'})).addTo(this);
+            var editInOSMControlOptions = {
+                position: 'topleft',
+                widgetOptions: {helpText: L._('Open this map extent in a map editor to provide more accurate data to OpenStreetMap')}
+            };
+            this._controls.editInOSMControl = (new L.Control.EditInOSM(editInOSMControlOptions)).addTo(this);
             var measureOptions = {
                 handler: {
                     icon: new L.DivIcon({
@@ -421,8 +445,11 @@ L.Storage.Map.include({
             north = parseFloat(this.options.limitBounds.north),
             east = parseFloat(this.options.limitBounds.east);
         if (!isNaN(south) && !isNaN(west) && !isNaN(north) && !isNaN(east)) {
-            this.setMaxBounds([[south, west], [north, east]]);
+            var bounds = L.latLngBounds([[south, west], [north, east]]);
+            this.options.minZoom = this.getBoundsZoom(bounds, true);
+            this.setMaxBounds(bounds);
         } else {
+            this.options.minZoom = 0;
             this.setMaxBounds();
         }
     },
@@ -456,33 +483,64 @@ L.Storage.Map.include({
     renderShareBox: function () {
         var container = L.DomUtil.create('div', 'storage-share'),
             embedTitle = L.DomUtil.add('h4', '', container, L._('Embed the map')),
-            iframe = L.DomUtil.create('textarea', 'storage-share-iframe', container),
-            url = window.location.protocol + '//' + window.location.host + window.location.pathname,
-            iframeUrl = url + '?scaleControl=0&miniMap=0&scrollWheelZoom=0&allowEdit=0';
-        iframe.innerHTML = '<iframe width="100%" height="300" frameBorder="0" src="' + iframeUrl +'"></iframe><p><a href="' + url + '">See full screen</a></p>';
+            iframe = L.DomUtil.create('textarea', 'storage-share-iframe', container);
+        var UIFields = [
+            ['dimensions.width', {handler: 'Input', label: L._('width')}],
+            ['dimensions.height', {handler: 'Input', label: L._('height')}],
+            ['options.includeFullScreenLink', {handler: 'CheckBox', helpText: L._('Include full screen link?')}],
+            ['options.currentView', {handler: 'CheckBox', helpText: L._('Current view instead of default map view?')}],
+            'queryString.moreControl',
+            'queryString.datalayersControl',
+            'queryString.zoomControl',
+            'queryString.scrollWheelZoom',
+            'queryString.miniMap',
+            'queryString.scaleControl',
+            'queryString.onLoadPanel'
+        ];
+        var iframeExporter = new L.S.IframeExporter(this);
+        var buildIframeCode = function () {
+            iframe.innerHTML = iframeExporter.build();
+        };
+        buildIframeCode();
+        var builder = new L.S.FormBuilder(iframeExporter, UIFields, {
+            callback: buildIframeCode
+        });
+        var iframeOptions = L.DomUtil.createFieldset(container, L._('Iframe export options'));
+        iframeOptions.appendChild(builder.build());
         if (this.options.shortUrl) {
+            L.DomUtil.create('hr', '', container);
             L.DomUtil.add('h4', '', container, L._('Short URL'));
             var shortUrl = L.DomUtil.create('input', 'storage-short-url', container);
             shortUrl.type = "text";
             shortUrl.value = this.options.shortUrl;
         }
+        L.DomUtil.create('hr', '', container);
         L.DomUtil.add('h4', '', container, L._('Download raw data (GeoJSON)'));
+        L.DomUtil.add('p', '', container, L._('Only visible features will be downloaded.'));
         var download = L.DomUtil.create('a', 'button', container);
+        download.innerHTML = L._('Download data');
+        download.download = "features.geojson";
+        L.DomEvent.on(download, 'click', function () {
+            var content = JSON.stringify(this.toGeoJSON(), null, 2);
+            window.URL = window.URL || window.webkitURL;
+            var blob = new Blob([content], {type: 'application/json'});
+            download.href = window.URL.createObjectURL(blob);
+        }, this);
+        L.S.fire('ui:start', {data:{html:container}});
+    },
+
+    toGeoJSON: function () {
         var features = [];
         this.eachDataLayer(function (datalayer) {
-            features = features.concat(datalayer.featuresToGeoJSON());
+            if (datalayer.isVisible()) {
+                features = features.concat(datalayer.featuresToGeoJSON());
+            }
         });
         var geojson = {
             type: "FeatureCollection",
             features: features
         };
-        var content = JSON.stringify(geojson, null, 2);
-        window.URL = window.URL || window.webkitURL;
-        var blob = new Blob([content], {type: 'application/json'});
-        download.href = window.URL.createObjectURL(blob);
-        download.innerHTML = L._('Download data');
-        download.download = "features.geojson";
-        L.S.fire('ui:start', {data:{html:container}});
+        return geojson;
     },
 
     updatePermissions: function () {
@@ -509,7 +567,7 @@ L.Storage.Map.include({
             layerLabel = L.DomUtil.create('label', '', container),
             submitInput = L.DomUtil.create('input', '', container),
             map = this, option,
-            types = ['geojson', 'csv', 'gpx', 'kml', 'osm'];
+            types = ['geojson', 'csv', 'gpx', 'kml', 'osm', 'georss'];
         title.innerHTML = L._('Import data');
         fileInput.type = "file";
         submitInput.type = "button";
@@ -526,15 +584,29 @@ L.Storage.Map.include({
         urlInput.placeholder = L._('Provide an URL here');
         rawInput.placeholder = L._('Paste here your data');
         this.eachDataLayer(function (datalayer) {
-            var id = L.stamp(datalayer);
-            option = L.DomUtil.create('option', '', layerInput);
-            option.value = id;
-            option.innerHTML = datalayer.options.name;
+            if (datalayer.isLoaded()) {
+                var id = L.stamp(datalayer);
+                option = L.DomUtil.create('option', '', layerInput);
+                option.value = id;
+                option.innerHTML = datalayer.options.name;
+            }
         });
         for (var i = 0; i < types.length; i++) {
             option = L.DomUtil.create('option', '', typeInput);
             option.value = option.innerHTML = types[i];
         }
+        if (layerInput.length === 0) {
+            var action = {
+                label: L._('Add a layer now'),
+                callback: function () {
+                    var datalayer = this._createDataLayer({});
+                    datalayer.edit();
+                },
+                callbackContext: this
+            };
+            L.S.fire('ui:alert', {content: L._('Please add at least a layer to import in'), level: 'info', duration: 30000, action: action});
+            return;
+        }
         if (this.options.importPresets.length) {
             var noPreset = L.DomUtil.create('option', '', presetSelect);
             noPreset.value = noPreset.innerHTML = L._('Choose a preset');
@@ -547,75 +619,27 @@ L.Storage.Map.include({
             presetBox.style.display = 'none';
         }
 
-        var processContent = function (raw) {
-            var type = typeInput.value,
-                layerId = layerInput[layerInput.selectedIndex].value;
-            layer = map.datalayers[layerId];
-            layer.addRawData(raw, type);
-            layer.isDirty = true;
-            L.S.fire('ui:end');
-            layer.zoomTo();
-        };
-
-        var detectTypeFromFile = function (f) {
-            var filename = f.name ? escape(f.name.toLowerCase()) : '';
-            function ext(_) {
-                return filename.indexOf(_) !== -1;
-            }
-            if (f.type === 'application/vnd.google-earth.kml+xml' || ext('.kml')) {
-                return 'kml';
-            }
-            if (ext('.gpx')) return 'gpx';
-            if (ext('.geojson') || ext('.json')) return 'geojson';
-            if (f.type === 'text/csv' || ext('.csv') || ext('.tsv') || ext('.dsv')) {
-                return 'dsv';
-            }
-            if (ext('.xml')) return 'osm';
-        };
-
-        var processFile = function () {
-            var files = fileInput.files,
-                output = [],
-                reader, content, geojson, layer;
-
-            var process = function (f) {
-                reader = new FileReader();
-                reader.readAsText(f);
-                reader.onload = function (e) {
-                    processContent(e.target.result);
-                };
-            };
-
-            for (var i = 0, f; f = files[i]; i++) {
-                process(f);
-            }
-        };
-
-        var processUrl = function (url) {
-            url = map.localizeUrl(url);
-            L.S.Xhr._ajax({verb: 'GET', uri: url, callback: function (data) {
-                processContent(data);
-            }});
-        };
-
         var submit = function () {
+            var type = typeInput.value,
+                layerId = layerInput[layerInput.selectedIndex].value,
+                layer = map.datalayers[layerId];
             if (fileInput.files) {
-                processFile();
+                layer.importFromFiles(fileInput.files);
             }
             if (rawInput.value) {
-                processContent(rawInput.value);
+                layer.importRaw(rawInput.value, type);
             }
             if (urlInput.value) {
-                processUrl(urlInput.value);
+                layer.importFromUrl(urlInput.value, type);
             }
-            if (presetSelect.selectedIndex !== 0) {
-                processUrl(presetSelect[presetSelect.selectedIndex].value);
+            if (presetSelect.selectedIndex > 0) {
+                layer.importFromUrl(presetSelect[presetSelect.selectedIndex].value, type);
             }
         };
         L.DomEvent.on(submitInput, 'click', submit, this);
         L.DomEvent.on(fileInput, 'change', function (e) {
             var f = e.target.files[0],
-                type = detectTypeFromFile(f);
+                type = L.Util.detectFileType(f);
             if (type) {
                 typeInput.value = type;
             }
@@ -625,37 +649,45 @@ L.Storage.Map.include({
 
     openBrowser: function () {
         this.whenReady(function () {
-            if (this._controls.datalayersControl) {
-            this._controls.datalayersControl.openBrowser();
-            }
+            this._openBrowser();
         });
     },
 
     displayCaption: function () {
         var container = L.DomUtil.create('div', 'storage-caption'),
-            title = L.DomUtil.create('h4', '', container);
+            title = L.DomUtil.create('h3', '', container);
         title.innerHTML = this.options.name;
+        if (this.options.author && this.options.author.name && this.options.author.link) {
+            var authorContainer = L.DomUtil.add('h5', 'storage-map-author', container, L._('by') + ' '),
+                author = L.DomUtil.create('a');
+            author.href = this.options.author.link;
+            author.innerHTML = this.options.author.name;
+            authorContainer.appendChild(author);
+        }
         if (this.options.description) {
             var description = L.DomUtil.create('div', 'storage-map-description', container);
             description.innerHTML = L.Util.toHTML(this.options.description);
         }
+        var datalayerContainer = L.DomUtil.create('div', 'datalayer-container', container);
         this.eachDataLayer(function (datalayer) {
-            var p = L.DomUtil.create('p', '', container),
-                color = L.DomUtil.create('span', 'datalayer_color', p),
-                title = L.DomUtil.create('strong', '', p),
+            var p = L.DomUtil.create('p', '', datalayerContainer),
+                color = L.DomUtil.create('span', 'datalayer-color', p),
+                headline = L.DomUtil.create('strong', '', p),
                 description = L.DomUtil.create('span', '', p);
-            if (datalayer.options.color) {
-                color.style.backgroundColor = datalayer.options.color;
-            }
-            title.innerHTML = datalayer.options.name + ' ';
-            if (datalayer.options.description) {
-                description.innerHTML = datalayer.options.description;
-            }
+                datalayer.onceLoaded(function () {
+                    color.style.backgroundColor = this.getColor();
+                    if (datalayer.options.description) {
+                        description.innerHTML = datalayer.options.description;
+                    }
+                });
+            datalayer.renderToolbox(headline);
+            L.DomUtil.add('span', '', headline, datalayer.options.name + ' ');
         });
-        L.DomUtil.create('hr', '', container);
-        title = L.DomUtil.create('h5', '', container);
+        var creditsContainer = L.DomUtil.create('div', 'credits-container', container),
+            credits = L.DomUtil.createFieldset(creditsContainer, L._('Credits'));
+        title = L.DomUtil.create('h5', '', credits);
         title.innerHTML = L._('User content credits');
-        var contentCredit = L.DomUtil.create('p', '', container),
+        var contentCredit = L.DomUtil.create('p', '', credits),
             licence = L.DomUtil.create('a', '', contentCredit);
         if (this.options.licence) {
             licence.innerHTML = this.options.licence.name;
@@ -663,16 +695,16 @@ L.Storage.Map.include({
             contentCredit.innerHTML =  L._('Map user content has been published under licence')
                                        + ' ' + contentCredit.innerHTML;
         }
-        L.DomUtil.create('hr', '', container);
-        title = L.DomUtil.create('h5', '', container);
+        L.DomUtil.create('hr', '', credits);
+        title = L.DomUtil.create('h5', '', credits);
         title.innerHTML = L._('Map background credits');
-        var tilelayerCredit = L.DomUtil.create('p', '', container),
+        var tilelayerCredit = L.DomUtil.create('p', '', credits),
             name = L.DomUtil.create('strong', '', tilelayerCredit),
             attribution = L.DomUtil.create('span', '', tilelayerCredit);
         name.innerHTML = this.selected_tilelayer.options.name + ' ';
         attribution.innerHTML = this.selected_tilelayer.getAttribution();
-        L.DomUtil.create('hr', '', container);
-        var umapCredit = L.DomUtil.create('p', '', container),
+        L.DomUtil.create('hr', '', credits);
+        var umapCredit = L.DomUtil.create('p', '', credits),
             urls = {
                 leaflet: 'http://leafletjs.com',
                 django: 'https://www.djangoproject.com',
@@ -692,17 +724,15 @@ L.Storage.Map.include({
     },
 
     reset: function () {
+        this.resetOptions();
         this.eachDataLayer(function (datalayer) {
-            if (datalayer.isDirty) {
-                datalayer.reset();
-            }
+            datalayer.reset();
         });
         this.deleted_datalayers.forEach(function (datalayer) {
             datalayer.connectToMap();
             datalayer.reset();
         });
         this.updateDatalayersControl();
-        this.resetOptions();
         this.initTileLayers();
         this.isDirty = false;
     },
@@ -745,19 +775,31 @@ L.Storage.Map.include({
         var editableOptions = [
             'zoom',
             'datalayersControl',
+            'scrollWheelZoom',
             'zoomControl',
             'scaleControl',
             'moreControl',
             'miniMap',
-            'displayCaptionOnLoad',
             'displayPopupFooter',
-            'displayDataBrowserOnLoad',
+            'onLoadPanel',
             'tilelayersControl',
             'name',
             'description',
             'licence',
             'tilelayer',
-            'limitBounds'
+            'limitBounds',
+            'color',
+            'iconClass',
+            'iconUrl',
+            'smoothFactor',
+            'opacity',
+            'weight',
+            'fill',
+            'fillColor',
+            'fillOpacity',
+            'dashArray',
+            'popupTemplate',
+            'captionBar'
         ], properties = {};
         for (var i = editableOptions.length - 1; i >= 0; i--) {
             if (typeof this.options[editableOptions[i]] !== "undefined") {
@@ -823,7 +865,7 @@ L.Storage.Map.include({
             }
         }
         if (datalayer && !datalayer.isRemoteLayer()) {
-            // No datalayer visibile, let's force one
+            // No datalayer visible, let's force one
             this.addLayer(datalayer.layer);
             return datalayer;
         }
@@ -857,24 +899,50 @@ L.Storage.Map.include({
         form = builder.build();
         container.appendChild(form);
         var UIFields = [
-            ['options.moreControl', {handler: 'CheckBox', helpText: L._("Do you want to display the 'more' control?")}],
-            ['options.datalayersControl', {handler: 'CheckBox', helpText: L._("Do you want to display the data layers control?")}],
-            ['options.zoomControl', {handler: 'CheckBox', helpText: L._("Do you want to display zoom control?")}],
-            ['options.scrollWheelZoom', {handler: 'CheckBox', helpText: L._("Allow scroll wheel zoom?")}],
-            ['options.miniMap', {handler: 'CheckBox', helpText: L._("Do you want to display a minimap?")}],
-            ['options.scaleControl', {handler: 'CheckBox', helpText: L._("Do you want to display the scale control?")}],
-            ['options.displayCaptionOnLoad', {handler: 'CheckBox', helpText: L._("Do you want to display map caption on load?")}],
-            ['options.displayDataBrowserOnLoad', {handler: 'CheckBox', helpText: L._("Do you want to display data browser on load?")}],
-            ['options.displayPopupFooter', {handler: 'CheckBox', helpText: L._("Do you want to display popup footer?")}]
+            'options.moreControl',
+            'options.datalayersControl',
+            'options.zoomControl',
+            'options.scrollWheelZoom',
+            'options.miniMap',
+            'options.scaleControl',
+            'options.onLoadPanel',
+            'options.displayPopupFooter',
+            'options.captionBar'
         ];
         builder = new L.S.FormBuilder(this, UIFields, {
             callback: this.initControls,
             callbackContext: this
         });
-        var controlsOptions = L.DomUtil.createFieldset(container, L._('Display options'));
+        var controlsOptions = L.DomUtil.createFieldset(container, L._('User interface options'));
         controlsOptions.appendChild(builder.build());
 
-        if (typeof this.options.tilelayer !== 'object') {
+        var optionsFields = [
+            'options.color',
+            'options.iconClass',
+            'options.iconUrl',
+            'options.smoothFactor',
+            'options.opacity',
+            'options.weight',
+            'options.fill',
+            'options.fillColor',
+            'options.fillOpacity',
+            'options.dashArray',
+            'options.popupTemplate',
+        ];
+
+        builder = new L.S.FormBuilder(this, optionsFields, {
+            callback: function (field) {
+                if (field !== "options.popupTemplate") {
+                    this.eachDataLayer(function (datalayer) {
+                        datalayer.redraw();
+                    });
+                }
+            }
+        });
+        var defaultProperties = L.DomUtil.createFieldset(container, L._('Default properties'));
+        defaultProperties.appendChild(builder.build());
+
+        if (!L.Util.isObject(this.options.tilelayer)) {
             this.options.tilelayer = {};
         }
         var tilelayerFields = [
@@ -882,7 +950,8 @@ L.Storage.Map.include({
             ['options.tilelayer.url_template', {handler: 'BlurInput', helpText: L._("Supported scheme") + ': http://{s}.domain.com/{z}/{x}/{y}.png', placeholder: 'url'}],
             ['options.tilelayer.maxZoom', {handler: 'BlurIntInput', placeholder: L._('max zoom')}],
             ['options.tilelayer.minZoom', {handler: 'BlurIntInput', placeholder: L._('min zoom')}],
-            ['options.tilelayer.attribution', {handler: 'BlurInput', placeholder: L._('attribution')}]
+            ['options.tilelayer.attribution', {handler: 'BlurInput', placeholder: L._('attribution')}],
+            ['options.tilelayer.tms', {handler: 'CheckBox', helpText: L._('TMS format')}]
         ];
         var customTilelayer = L.DomUtil.createFieldset(container, L._('Custom background'));
         builder = new L.S.FormBuilder(this, tilelayerFields, {
@@ -891,7 +960,7 @@ L.Storage.Map.include({
         });
         customTilelayer.appendChild(builder.build());
 
-        if (typeof this.options.limitBounds !== 'object' || this.options.limitBounds === null) {
+        if (!L.Util.isObject(this.options.limitBounds)) {
             this.options.limitBounds = {};
         }
         var limitBounds = L.DomUtil.createFieldset(container, L._('Limit bounds'));
@@ -910,10 +979,10 @@ L.Storage.Map.include({
         setCurrentButton.href = "#";
         L.DomEvent.on(setCurrentButton, 'click', function (e) {
             var bounds = this.getBounds();
-            this.options.limitBounds.south = bounds.getSouth();
-            this.options.limitBounds.west = bounds.getWest();
-            this.options.limitBounds.north = bounds.getNorth();
-            this.options.limitBounds.east = bounds.getEast();
+            this.options.limitBounds.south = L.Util.formatNum(bounds.getSouth());
+            this.options.limitBounds.west = L.Util.formatNum(bounds.getWest());
+            this.options.limitBounds.north = L.Util.formatNum(bounds.getNorth());
+            this.options.limitBounds.east = L.Util.formatNum(bounds.getEast());
             boundsBuilder.fetchAll();
             this.isDirty = true;
             this.handleLimitBounds();
@@ -926,7 +995,7 @@ L.Storage.Map.include({
         L.DomEvent
             .on(del, 'click', L.DomEvent.stop)
             .on(del, 'click', this.del, this);
-        var clone = L.DomUtil.create('a', 'clone-map-button', advancedActions);
+        var clone = L.DomUtil.create('a', 'storage-clone', advancedActions);
         clone.href = "#";
         clone.innerHTML = L._('Clone this map');
         L.DomEvent
@@ -945,12 +1014,39 @@ L.Storage.Map.include({
         this.editEnabled = false;
     },
 
+    getDisplayName: function () {
+        return this.options.name || L._('Untitled map');
+    },
+
+    initCaptionBar: function () {
+        var container = L.DomUtil.create('div', 'storage-caption-bar', this._controlContainer),
+            name = L.DomUtil.create('h3', '', container);
+        if (this.options.author && this.options.author.name && this.options.author.link) {
+            var authorContainer = L.DomUtil.add('span', 'storage-map-author', container, ' ' + L._('by') + ' '),
+                author = L.DomUtil.create('a');
+            author.href = this.options.author.link;
+            author.innerHTML = this.options.author.name;
+            authorContainer.appendChild(author);
+        }
+        var about = L.DomUtil.add('a', 'storage-about-link', container, ' — ' + L._('About'));
+        about.href = '#';
+        L.DomEvent.on(about, 'click', this.displayCaption, this);
+        var browser = L.DomUtil.add('a', 'storage-open-browser-link', container, ' | ' + L._('Browse data'));
+        browser.href = '#';
+        L.DomEvent.on(browser, 'click', this.openBrowser, this);
+        var setName = function () {
+            name.innerHTML = this.getDisplayName();
+        };
+        L.bind(setName, this)();
+        this.on('synced', L.bind(setName, this));
+    },
+
     initEditBar: function () {
         var container = L.DomUtil.create('div', 'storage-main-edit-toolbox', this._controlContainer),
             title = L.DomUtil.add('h3', '', container, L._("Editing") + '&nbsp;'),
             name = L.DomUtil.create('a', 'storage-click-to-edit', title),
             setName = function () {
-                name.innerHTML = this.options.name || L._('Untitled map');
+                name.innerHTML = this.getDisplayName();
             };
         L.bind(setName, this)();
         L.DomEvent.on(name, 'click', this.edit, this);
@@ -1075,19 +1171,22 @@ L.Storage.Map.include({
     },
 
     setContextMenuItems: function (e) {
-        var items = [
-            {
+        var items = [];
+        if (this._zoom !== this.getMaxZoom()) {
+            items.push({
                 text: L._('Zoom in'),
                 callback: function () {this.zoomIn();}
-            },
-            {
+            });
+        }
+        if (this._zoom !== this.getMinZoom()) {
+            items.push({
                 text: L._('Zoom out'),
                 callback: function () {this.zoomOut();}
-            }
-        ];
+            });
+        }
         if (e && e.relatedTarget) {
             if (e.relatedTarget.getContextMenuItems) {
-                items = items.concat(e.relatedTarget.getContextMenuItems());
+                items = items.concat(e.relatedTarget.getContextMenuItems(e));
             }
         }
         if (this.options.allowEdit) {
