@@ -1,3 +1,138 @@
+L.S.Layer = {
+    isBrowsable: true,
+
+    getFeatures: function () {
+        return this._layers;
+    },
+
+    eachFeature: function (method, context) {
+        var features = this.getFeatures();
+        for (var i in features) {
+            method.call(context || this, features[i]);
+        }
+    },
+
+    getEditableOptions: function () {return [];},
+
+    postUpdate: function (field) {}
+
+};
+
+L.S.Layer.Default = L.FeatureGroup.extend({
+    _type: "Default",
+    includes: [L.S.Layer],
+
+    initialize: function (datalayer) {
+        this.datalayer = datalayer;
+        L.FeatureGroup.prototype.initialize.call(this);
+    }
+
+});
+
+L.S.Layer.Cluster = L.MarkerClusterGroup.extend({
+    _type: "Cluster",
+    includes: [L.S.Layer],
+
+    initialize: function (datalayer) {
+        this.datalayer = datalayer;
+        var options = {
+            polygonOptions: {
+                color: this.datalayer.getColor()
+            },
+            iconCreateFunction: function (cluster) {
+                return new L.Storage.Icon.Cluster(datalayer, cluster);
+            }
+        };
+        if (this.datalayer.options.cluster && this.datalayer.options.cluster.radius) {
+            options.maxClusterRadius = this.datalayer.options.cluster.radius;
+        }
+        L.MarkerClusterGroup.prototype.initialize.call(this, options);
+    },
+
+    getEditableOptions: function () {
+        if (!L.Util.isObject(this.datalayer.options.cluster)) {
+            this.datalayer.options.cluster = {};
+        }
+        return [
+            ['options.cluster.radius', {handler: 'BlurIntInput', placeholder: L._('Clustering radius'), helpText: L._('Override clustering radius (default 80)')}],
+        ];
+
+    },
+
+    postUpdate: function (field) {
+        if (field === "options.cluster.radius") {
+            // No way to reset radius of an already instanciated MarkerClusterGroup...
+            this.datalayer.resetLayer(true);
+            return;
+        }
+        if (field === "options.color") {
+            this.options.polygonOptions.color = this.datalayer.getColor();
+        }
+    },
+
+    eachFeature: function (method, context) {
+        this.eachLayer(method, context);
+    }
+
+});
+
+L.S.Layer.Heat = L.HeatLayer.extend({
+    _type: "Heat",
+    includes: [L.S.Layer],
+    isBrowsable: false,
+
+    initialize: function (datalayer) {
+        this.datalayer = datalayer;
+        L.HeatLayer.prototype.initialize.call(this, [], this.datalayer.options.heat);
+    },
+
+    addLayer: function (layer) {
+        if (layer instanceof L.Marker) {
+            var latlng = layer.getLatLng(), alt;
+            if (this.datalayer.options.heat && this.datalayer.options.heat.intensityProperty) {
+                alt = parseFloat(layer.properties[this.datalayer.options.heat.intensityProperty || 0]);
+                latlng = new L.LatLng(latlng.lat, latlng.lng, alt);
+            }
+            this.addLatLng(latlng);
+        }
+    },
+
+    clearLayers: function () {
+        this.setLatLngs([]);
+    },
+
+    getFeatures: function () {
+        return {};
+    },
+
+    getBounds: function () {
+        return L.latLngBounds(this._latlngs);
+    },
+
+    getEditableOptions: function () {
+        if (!L.Util.isObject(this.datalayer.options.heat)) {
+            this.datalayer.options.heat = {};
+        }
+        return [
+            ['options.heat.radius', {handler: 'BlurIntInput', placeholder: L._('Heatmap radius'), helpText: L._('Override heatmap radius (default 25)')}],
+            ['options.heat.intensityProperty', {handler: 'BlurInput', placeholder: L._('Heatmap intensity property'), helpText: L._('Optional intensity property for heatmap')}],
+        ];
+
+    },
+
+    postUpdate: function (field) {
+        if (field === 'options.heat.intensityProperty') {
+            this.datalayer.resetLayer(true);  // We need to repopulate the latlngs
+            return;
+        }
+        if (field === 'options.heat.radius') {
+            this.options.radius = this.datalayer.options.heat.radius;
+        }
+        this._updateOptions();
+    }
+
+});
+
 L.Storage.DataLayer = L.Class.extend({
 
     includes: [L.Mixin.Events],
@@ -35,7 +170,7 @@ L.Storage.DataLayer = L.Class.extend({
         this.populate(datalayer);
         this.connectToMap();
         if(this.options.displayOnLoad) {
-            this.display();
+            this.show();
         }
         if (!this.storage_id) {
             this.isDirty = true;
@@ -49,11 +184,14 @@ L.Storage.DataLayer = L.Class.extend({
         });
     },
 
-    resetLayer: function () {
-        if (this.layer && ((this.layer instanceof L.MarkerClusterGroup && this.isClustered()) ||
-            (!this.isClustered() && !(this.layer instanceof L.MarkerClusterGroup)))) {
-            return;
+    resetLayer: function (force) {
+        // Backward compat, to be removed
+        if (this.options.markercluster) {
+            this.options.type = "Cluster";
+            delete this.options.markercluster;
         }
+
+        if (this.layer && this.options.type === this.layer._type && !force) return;
         var visible = this.isVisible(), self = this;
         if (visible) {
             this.map.removeLayer(this.layer);
@@ -61,18 +199,8 @@ L.Storage.DataLayer = L.Class.extend({
         if (this.layer) {
             this.layer.clearLayers();
         }
-        if (this.isClustered()) {
-            this.layer = new L.MarkerClusterGroup({
-                polygonOptions: {
-                    color: this.options.color || this.map.getDefaultOption('color')
-                },
-                iconCreateFunction: function (cluster) {
-                    return new L.Storage.Icon.Cluster(self, cluster);
-                }
-            });
-        } else {
-            this.layer = new L.FeatureGroup();
-        }
+        var Class = L.S.Layer[this.options.type] || L.S.Layer.Default;
+        this.layer = new Class(this);
         this.eachLayer(function (layer) {
             this.layer.addLayer(layer);
         });
@@ -88,6 +216,13 @@ L.Storage.DataLayer = L.Class.extend({
         return this;
     },
 
+    eachFeature: function (method, context) {
+        if (this.layer) {
+            this.layer.eachFeature(method, context | this);
+        }
+        return this;
+    },
+
     fetchData: function () {
         if (!this.storage_id) {
             return;
@@ -102,6 +237,8 @@ L.Storage.DataLayer = L.Class.extend({
                 } else {
                     this.fromGeoJSON(geojson);
                 }
+                this._loaded = true;
+                this.fire('loaded');
             },
             context: this
         });
@@ -111,24 +248,34 @@ L.Storage.DataLayer = L.Class.extend({
         this.addData(geojson);
         this._geojson = geojson;
         this.fire('dataloaded');
+        this.fire('datachanged');
     },
 
     clear: function () {
         this.layer.clearLayers();
         this._layers = {};
+        this._index = Array();
+        if (this._geojson) {
+            this._geojson_bk = this._geojson;
+            this._geojson = null;
+        }
     },
 
     fetchRemoteData: function () {
-        if ((!isNaN(this.options.remoteData.from) && this.map.getZoom() < this.options.remoteData.from) ||
-            (!isNaN(this.options.remoteData.to) && this.map.getZoom() > this.options.remoteData.to) ) {
+        var from = parseInt(this.options.remoteData.from, 10),
+            to = parseInt(this.options.remoteData.to, 10);
+        if ((!isNaN(from) && this.map.getZoom() < from) ||
+            (!isNaN(to) && this.map.getZoom() > to) ) {
             this.clear();
             return;
         }
-        var self = this;
-        this._geojson = {}; // Should appear as loaded (and so editable) even if xhr goes in error
-                            // du to user missconfigurating the remote URL
+        var self = this,
+            url = this.map.localizeUrl(this.options.remoteData.url);
+        if (this.options.remoteData.proxy && this.map.options.urls.ajax_proxy) {
+            url = L.Util.template(this.map.options.urls.ajax_proxy, {url: encodeURIComponent(url)});
+        }
         this.map.ajax({
-            uri: this.map.localizeUrl(this.options.remoteData.url),
+            uri: url,
             verb: 'GET',
             callback: function (raw) {
                 self.clear();
@@ -140,6 +287,15 @@ L.Storage.DataLayer = L.Class.extend({
     onceLoaded: function (callback, context) {
         if (this.isLoaded()) {
             callback.call(context || this, this);
+        } else {
+            this.once('loaded', callback, context);
+        }
+        return this;
+    },
+
+    onceDataLoaded: function (callback, context) {
+        if (this.hasDataLoaded()) {
+            callback.call(context || this, this);
         } else {
             this.once('dataloaded', callback, context);
         }
@@ -147,6 +303,10 @@ L.Storage.DataLayer = L.Class.extend({
     },
 
     isLoaded: function () {
+        return !this.storage_id || this._loaded;
+    },
+
+    hasDataLoaded: function () {
         return !this.storage_id || this._geojson !== null;
     },
 
@@ -175,7 +335,7 @@ L.Storage.DataLayer = L.Class.extend({
 
     _dataUrl: function() {
         var template = this.map.options.urls.datalayer_view;
-        return L.Util.template(template, {"pk": this.storage_id});
+        return L.Util.template(template, {"pk": this.storage_id, "map_id": this.map.options.storage_id});
     },
 
     isRemoteLayer: function () {
@@ -183,7 +343,7 @@ L.Storage.DataLayer = L.Class.extend({
     },
 
     isClustered: function () {
-        return !!this.options.markercluster;
+        return this.options.type === "Cluster";
     },
 
     addLayer: function (feature) {
@@ -192,6 +352,9 @@ L.Storage.DataLayer = L.Class.extend({
         this._index.push(id);
         this._layers[id] = feature;
         this.layer.addLayer(feature);
+        if (this.hasDataLoaded()) {
+            this.fire('datachanged');
+        }
     },
 
     removeLayer: function (feature) {
@@ -200,6 +363,9 @@ L.Storage.DataLayer = L.Class.extend({
         this._index.splice(this._index.indexOf(id), 1);
         delete this._layers[id];
         this.layer.removeLayer(feature);
+        if (this.hasDataLoaded()) {
+            this.fire('datachanged');
+        }
     },
 
     addData: function (geojson) {
@@ -233,10 +399,18 @@ L.Storage.DataLayer = L.Class.extend({
             });
         } else if (type === 'gpx') {
             callback(toGeoJSON.gpx(toDom(c)));
+        } else if (type === 'georss') {
+            callback(GeoRSSToGeoJSON(toDom(c)));
         } else if (type === 'kml') {
             callback(toGeoJSON.kml(toDom(c)));
         } else if (type === 'osm') {
-            callback(osm_geojson.osm2geojson(toDom(c)));
+            var d;
+            try {
+                d = JSON.parse(c);
+            } catch (e) {
+                d = toDom(c);
+            }
+            callback(osmtogeojson(d, {flatProperties: true}));
         } else if (type === "geojson") {
             try {
                 gj = JSON.parse(c);
@@ -254,7 +428,9 @@ L.Storage.DataLayer = L.Class.extend({
 
         if (features) {
             features.sort(function (a, b) {
-                if (!a.properties.name && ! b.properties.name) {
+                if (!a.properties || !b.properties) {
+                    return 0;
+                } else if (!a.properties.name && !b.properties.name) {
                     return 0;
                 } else if (!a.properties.name) {
                     return -1;
@@ -269,8 +445,9 @@ L.Storage.DataLayer = L.Class.extend({
             return this;
         }
 
-        var geometry = geojson.type === 'Feature' ? geojson.geometry : geojson,
-            coords = geometry.coordinates,
+        var geometry = geojson.type === 'Feature' ? geojson.geometry : geojson;
+        if (!geometry) return;  // null geometry is valid geojson.
+        var coords = geometry.coordinates,
             layer;
 
         switch (geometry.type) {
@@ -293,6 +470,8 @@ L.Storage.DataLayer = L.Class.extend({
                     layer = this._polygonToLayer(geojson, latlngs);
                     break;
                 }
+            case 'GeometryCollection':
+                return this.geojsonToFeatures(geojson.geometries);
             default:
                 L.S.fire('ui:alert', {content: L._("Skipping unkown geometry.type: {type}", {type: geometry.type}), level: 'error'});
         }
@@ -329,6 +508,35 @@ L.Storage.DataLayer = L.Class.extend({
         );
     },
 
+    importRaw: function (raw, type) {
+        this.addRawData(raw, type);
+        this.isDirty = true;
+        this.zoomTo();
+    },
+
+    importFromFiles: function (files) {
+        for (var i = 0, f; f = files[i]; i++) {
+            this.importFromFile(f);
+        }
+    },
+
+    importFromFile: function (f) {
+        var reader = new FileReader(),
+            type = L.Util.detectFileType(f),
+            self = this;
+        reader.readAsText(f);
+        reader.onload = function (e) {
+            self.importRaw(e.target.result, type);
+        };
+    },
+
+    importFromUrl: function (url, type) {
+        url = this.map.localizeUrl(url), self = this;
+        L.S.Xhr._ajax({verb: 'GET', uri: url, callback: function (data) {
+            self.importRaw(data, type);
+        }});
+    },
+
     getEditUrl: function() {
         return L.Util.template(this.map.options.urls.datalayer_update, {'map_id': this.map.options.storage_id, 'pk': this.storage_id});
     },
@@ -341,16 +549,8 @@ L.Storage.DataLayer = L.Class.extend({
         return (this.storage_id && this.getEditUrl()) || this.getCreateUrl();
     },
 
-    getIconClass: function () {
-        var iconClass = this.map.getDefaultOption('iconClass');
-        if(L.Storage.Icon[this.options.iconClass]) {
-            iconClass = this.options.iconClass;
-        }
-        return iconClass;
-    },
-
-    getIcon: function () {
-        return new L.Storage.Icon[this.getIconClass()](this.map);
+    getColor: function () {
+        return this.options.color || this.map.getDefaultOption('color');
     },
 
     getDeleteUrl: function () {
@@ -366,26 +566,49 @@ L.Storage.DataLayer = L.Class.extend({
         this.isDirty = true;
     },
 
+    empty: function () {
+        if (this.isRemoteLayer()) return;
+        this.clear();
+        this.isDirty = true;
+    },
+
+    clone: function () {
+        var options = L.Util.CopyJSON(this.options);
+        options.name = L._('Clone of {name}', {name: this.options.name});
+        delete options.id;
+        var geojson = L.Util.CopyJSON(this._geojson),
+            datalayer = this.map._createDataLayer(options);
+        datalayer.fromGeoJSON(geojson);
+        return datalayer;
+    },
+
     erase: function () {
         this.hide();
         delete this.map.datalayers[L.stamp(this)];
         this.map.datalayers_index.splice(this.map.datalayers_index.indexOf(this), 1);
         this.map.updateDatalayersControl();
-        this._layers = {};
-        this._index = Array();
+        this.fire('erase');
+        this._leaflet_events_bk = this._leaflet_events;
+        this.off();
+        this.clear();
+        delete this._loaded;
     },
 
     reset: function () {
         if (this.storage_id) {
+            if (this._leaflet_events_bk && !this._leaflet_events) {
+                this._leaflet_events = this._leaflet_events_bk;
+            }
             this.hide();
             this.clear();
             if (this.isRemoteLayer()) {
                 this.fetchRemoteData();
-            } else if (this._geojson) {
-                this.resetOptions(this._geojson._storage);
-                this.fromGeoJSON(this._geojson);
+            } else if (this._geojson_bk) {
+                this.resetOptions(this._geojson_bk._storage);
+                this.fromGeoJSON(this._geojson_bk);
             }
-            this.display();
+            this._loaded = true;
+            this.show();
             this.isDirty = false;
         } else {
             this.erase();
@@ -394,7 +617,7 @@ L.Storage.DataLayer = L.Class.extend({
 
     redraw: function () {
         this.hide();
-        this.display();
+        this.show();
     },
 
     edit: function () {
@@ -404,10 +627,17 @@ L.Storage.DataLayer = L.Class.extend({
             metadata_fields = [
                 'options.name',
                 'options.description',
+                ['options.type', {handler: 'LayerTypeChooser', label: L._('Type of layer')}],
                 ['options.displayOnLoad', {label: L._('Display on load'), handler: 'CheckBox'}]
             ];
         var builder = new L.S.FormBuilder(this, metadata_fields, {
-            callback: function () { this.map.updateDatalayersControl(); },
+            callback: function (field) {
+                this.map.updateDatalayersControl();
+                if (field === "options.type") {
+                    this.resetLayer();
+                    this.edit();
+                }
+            },
             callbackContext: this
         });
         form = builder.build();
@@ -424,28 +654,24 @@ L.Storage.DataLayer = L.Class.extend({
             'options.fillColor',
             'options.fillOpacity',
             'options.dashArray',
-            'options.popupTemplate',
-            ['options.markercluster', {handler: 'CheckBox', label: L._('Cluster markers')}],
+            'options.popupTemplate'
         ];
 
+        optionsFields = optionsFields.concat(this.layer.getEditableOptions());
+
         builder = new L.S.FormBuilder(this, optionsFields, {
             id: 'datalayer-advanced-properties',
             callback: function (field) {
                 this.hide();
-                if (field === "options.markercluster") {
-                    this.resetLayer();
-                }
-                if (field === "options.color" && this.isClustered()) {
-                    this.layer.options.polygonOptions.color = this.options.color || this.map.getDefaultOption('color');
-                }
-                this.display();
+                this.layer.postUpdate(field);
+                this.show();
             }
         });
         var advancedProperties = L.DomUtil.createFieldset(container, L._('Advanced properties'));
         form = builder.build();
         advancedProperties.appendChild(form);
 
-        if (typeof this.options.remoteData === "undefined") {
+        if (!L.Util.isObject(this.options.remoteData)) {
             this.options.remoteData = {};
         }
         var remoteDataFields = [
@@ -456,13 +682,16 @@ L.Storage.DataLayer = L.Class.extend({
             ['options.remoteData.dynamic', {handler: 'CheckBox', label: L._('Dynamic')}],
             ['options.remoteData.licence', {label: L._('Licence'), helpText: L._('Please be sure the licence is compliant with your use.')}]
         ];
+        if (this.map.options.urls.ajax_proxy) {
+            remoteDataFields.push(['options.remoteData.proxy', {handler: 'CheckBox', label: L._('Proxy request'), helpText: L._('To use if remote server doesn\'t allow cross domain (slower)')}]);
+        }
 
         var remoteDataContainer = L.DomUtil.createFieldset(container, L._('Remote data'));
         builder = new L.S.FormBuilder(this, remoteDataFields);
         remoteDataContainer.appendChild(builder.build());
 
         var advancedActions = L.DomUtil.createFieldset(container, L._('Advanced actions'));
-        var deleteLink = L.DomUtil.create('a', 'delete_datalayer_button', advancedActions);
+        var deleteLink = L.DomUtil.create('a', 'delete_datalayer_button storage-delete', advancedActions);
         deleteLink.innerHTML = L._('Delete');
         deleteLink.href = "#";
         L.DomEvent.on(deleteLink, 'click', L.DomEvent.stop)
@@ -470,6 +699,21 @@ L.Storage.DataLayer = L.Class.extend({
                     this._delete();
                     L.S.fire('ui:end');
                 }, this);
+        if (!this.isRemoteLayer()) {
+            var emptyLink = L.DomUtil.create('a', 'storage-empty', advancedActions);
+            emptyLink.innerHTML = L._('Empty');
+            emptyLink.href = "#";
+            L.DomEvent.on(emptyLink, 'click', L.DomEvent.stop)
+                      .on(emptyLink, 'click', this.empty, this);
+        }
+        var cloneLink = L.DomUtil.create('a', 'storage-clone', advancedActions);
+        cloneLink.innerHTML = L._('Clone');
+        cloneLink.href = "#";
+        L.DomEvent.on(cloneLink, 'click', L.DomEvent.stop)
+                  .on(cloneLink, 'click', function () {
+                    datalayer = this.clone();
+                    datalayer.edit();
+                }, this);
         L.S.fire('ui:start', {data: {html: container}});
 
     },
@@ -482,24 +726,22 @@ L.Storage.DataLayer = L.Class.extend({
         return features;
     },
 
-    display: function () {
+    show: function () {
         if(!this.isLoaded()) {
             this.fetchData();
         }
         this.map.addLayer(this.layer);
-        this.fire('display');
-        // this._map.fire('overlayadd', {layer: obj});
+        this.fire('show');
     },
 
     hide: function () {
         this.map.removeLayer(this.layer);
         this.fire('hide');
-        // this._map.fire('overlayremove', {layer: obj});
     },
 
     toggle: function () {
         if (!this.isVisible()) {
-            this.display();
+            this.show();
         }
         else {
             this.hide();
@@ -544,7 +786,7 @@ L.Storage.DataLayer = L.Class.extend({
     getNextVisible: function () {
         var id = this.map.datalayers_index.indexOf(this),
             next = this.map.datalayers_index[id + 1] || this.map.datalayers_index[0];
-        while(!next.isVisible() || next._index.length === 0) {
+        while(!next.isVisible() || next._index.length === 0 || !next.isBrowsable()) {
             next = next.getNextVisible();
         }
         return next;
@@ -553,12 +795,16 @@ L.Storage.DataLayer = L.Class.extend({
     getPreviousVisible: function () {
         var id = this.map.datalayers_index.indexOf(this),
             prev = this.map.datalayers_index[id - 1] || this.map.datalayers_index[this.map.datalayers_index.length - 1];
-        while(!prev.isVisible() || prev._index.length === 0) {
+        while(!prev.isVisible() || prev._index.length === 0 || !prev.isBrowsable()) {
             prev = prev.getPreviousVisible();
         }
         return prev;
     },
 
+    isBrowsable: function () {
+        return this.layer && this.layer.isBrowsable;
+    },
+
     save: function () {
         if (!this.isLoaded()) {return;}
         var geojson = {
@@ -606,7 +852,8 @@ L.TileLayer.include({
             maxZoom: this.options.maxZoom,
             attribution: this.options.attribution,
             url_template: this._url,
-            name: this.options.name
+            name: this.options.name,
+            tms: this.options.tms
         };
     },
 
@@ -614,4 +861,4 @@ L.TileLayer.include({
         return L.Util.toHTML(this.options.attribution);
     }
 
-});
\ No newline at end of file
+});
